/* util.c
 * S.Coffin	GV Computing	1/93
 *
 * Much reduced since goprocs.c added :-)
 */

/*  Copyright (c) 1992,1993,1994,1995 by Stephen Coffin.  All rights reserved.
 *
 *  This program is distributed in the hope that it will be useful.
 *  Use and copying of this software and preparation of derivative works
 *  based upon this software are permitted, so long as the following
 *  conditions are met:
 *       o credit to the authors is acknowledged following current
 *         academic behavior
 *       o no fees or compensation are charged for use, copies, or
 *         access to this software
 *	 o no fees or compensation are charged for access to the
 *	   game server for which this software is a client
 *       o this copyright notice is included intact.
 *  This software is made available AS IS, and no warranty is made about 
 *  the software or its performance. 
 * 
 *  Bug descriptions, use reports, comments or suggestions are welcome.
 *  Send them to    scoffin@netcom.com
 */

#include <stdio.h>
#include <time.h>
#include "shared.h"
#include "filter.h"
#include "goprocs.h"

extern int boardchanged;
extern int beep();
extern void set_prisoners();

int prisoners[2];

extern game maingame;
extern game stepgame;

#define piece unsigned char

static int boardsize = 19;

initboard( size )
	unsigned char size;
{
	boardsize = size;
	initialize(&maingame,size);
	maingame.gametype = REALGAME;
	maingame.sgf.gametype[0] = REALGAME;
	set_prisoners( 0, 0 );

	initialize(&stepgame,size);
	stepgame.gametype = STEPGAME;
	stepgame.sgf.gametype[0] = STEPGAME;
	stepgame.stoneNumber = Maxhistory;	/* Show stepgame not in use */
}

unsigned char *stamptime() {
	extern char *asctime();                /* stuff to get timestamp */
	struct tm *localtime(), *tp;
	struct timeval tv;
	struct timezone tz;

	gettimeofday( &tv, &tz );                /* fill in timestamp */
	tp = localtime( (time_t *)&tv.tv_sec );
	return( (unsigned char *)asctime(tp) );
}

/* save game in sgf format */
save_game_sgf( s, p1, p2, hcap, r1, r2, komi, type )
	unsigned char *s;	/* file name to save to */
	unsigned char *p1, *p2;	/* player names */
	int hcap;	/* handicap stones or 0 */
	float komi;	/* komi */
	unsigned char *r1, *r2;
	int type;	/* save REALMOVES or STEPMOVES */
{
	FILE *fp;
	int i, x, y;
	unsigned char *str;

	int s1;
	game *g;

	if( type == REALMOVE ) {
	    g = &maingame;
	}
	else if( type == STEPMOVE ) {
	    g = &stepgame;
	}

	if( (fp = fopen( (char *)s, "a+" )) == NULL ) {
		perror( "savegame" );
		return( 0 );
	}
	fprintf( fp, "(;\nPB[%s]BR[%s]\nPW[%s]WR[%s]\n", p2, r2, p1, r1 );
	str = stamptime();
	str[strlen((char *)str)-1] = '\0';
	fprintf( fp, "GM[1]VW[]EV[]DT[%s]SZ[%d]PC[IGS]KM[%4.1f]",
			str, boardsize, komi );
	if( hcap ) fprintf( fp, "HA[%d]", hcap );
	fprintf( fp, "\nUS[File created by xigc]\n" );

	if( hcap != 0 ) {
	    sgf_hcap( fp, boardsize, hcap );
	    i = 1;
	}

	fprintf( fp, ";\n" );

	for(i = 1; i <= g->stoneNumber; i++ ) {
	  if (g->history[i] < 0) {	/* pass */
	    fprintf(fp, "%s[tt];", 
	                (g->sgf.typehistory[i] & 1) ? "W" : "B");
	  } else if (g->sgf.typehistory[i] < Pass) {
	    s1 = g->history[i];
	    fprintf(fp, "%s[%c%c];",
	      (g->sgf.typehistory[i] & 1) ? "W" : "B",
	      'a' + (unsigned char) g->rowbrd[s1],
	      'a' + (unsigned char) boardsize - g->colbrd[s1] - 1);
	  }
	  if( !(i % 10 )) fprintf( fp, "\n" );
	}

	fprintf( fp, ")\n" );

	fclose( fp );
	return( 1 );
}

/* save the moves in a file */
save_game( s, p1, p2, hcap, type )
	unsigned char *s;	/* file name to save to */
	unsigned char *p1, *p2;	/* player names */
	int hcap;	/* handicap stones or 0 */
	int type;	/* save REALMOVE or STEPMOVE */
{
	FILE *fp;
	int i, x, y;
	int m;

        int s1;
	game *g;

	if( type == REALMOVE ) {
	   g = &maingame;
	}
	else if( type == STEPMOVE ) {
	   g = &stepgame;
	}

	if( (fp = fopen( (char *)s, "a+" )) == NULL ) {
		perror( "savegame" );
		return( 0 );
	}

	fprintf( fp, "\n%s\n", stamptime() );
	fprintf( fp, "White: %s\nBlack: %s\n", p1, p2 );
	fprintf( fp, "Handicap: %d\n\n", hcap );

	if( hcap != 0 ) {
	    fprintf( fp, "  0 (B): Handicap %d\n", hcap );
	    i = 1;
	}
	else i = 0;

	for(m = 1; m <= g->stoneNumber; m++ ) {
	  s1 = g->history[m];		/* 1 dimensional stone position */
	  if (s1 < 0) {	/* pass */
	    fprintf( fp, "%3d (%c):  Pass\n", i, 
		(g->sgf.typehistory[m] & 1) ? 'W' : 'B' );
	    i++;
	  } else if (g->sgf.typehistory[m] < Pass) {
	      fprintf( fp, "%3d (%c):  ", i, 
		  (g->sgf.typehistory[m] & 1) ? 'W' : 'B' );
	      x = g->rowbrd[s1] + 1;
	      fprintf( fp, "%c%d\n", 
		  'A' + ((x > 8) ? x + 1 : x) - 1, 
		  g->colbrd[s1] + 1 );
	      i++;
	  }
	}
	fclose( fp );
	return( 1 );
}

void get_prisoners( x, y )
	int *x, *y;
{
	*x = prisoners[0];
	*y = prisoners[1];
}

void set_prisoners( x, y )
	int x, y;
{
	prisoners[0] = x;
	prisoners[1] = y;
}

void update_prisoners( x, y )
	int x, y;
{
	prisoners[0] += x;
	prisoners[1] += y;
}

void score_it( i, j )
	int i, j;
{
	int s1;

	boardchanged = TRUE;
	if (maingame.stoneNumber && (stepgame.stoneNumber >= Maxhistory)) {
	  stepgame = maingame;
	  stepgame.gametype = STEPGAME;
	}

	s1 = j-1+(maingame.width*(i-1));
	if (stepgame.board[s1] != Blank) 
	  removedead(&stepgame,s1,DRAWIT);
	else
	  beep();
}

static int verts[3][26] = {
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 
	2, 0, 0, 0, 3, 0, 0, 0, 0, 0,  3, 0,  3, 0, 0, 0, 4},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 
	4, 0, 0, 0, 6, 0, 0, 0, 0, 0,  9, 0, 10, 0, 0, 0, 12},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 
	6, 0, 0, 0, 9, 0, 0, 0, 0, 0, 15, 0, 17, 0, 0, 0, 20},
};

/*
 * Set the handicap of "hand" stones on the board....
 * use "verts" table above....
 */
sethandicap( hand, type )
	int hand;
	int type;
{
	int top, mid, bot;
	piece t;

	top = verts[TOP][boardsize] +1;
	mid = verts[MID][boardsize] +1;
	bot = verts[BOT][boardsize] +1;

	/* 
	 * All of these drop through, '5' checks for odd points.
	 */
	t=1;
	switch (hand) {
	    case 9: 		/* taken care of by handi == 5 */
	    case 8: 
		makehandicap( mid, top, 0, t, type, DRAWIT );
		makehandicap( mid, bot, 0, t, type, DRAWIT );
	    case 7: 		/* taken care of by handi == 5 */
	    case 6: 
		makehandicap( bot, mid, 0, t, type, DRAWIT );
		makehandicap( top, mid, 0, t, type, DRAWIT );
	    case 5: 
		/* odd points, test because of drop through */
		if (hand & 0x1) {
		    makehandicap( mid, mid, 0, t, type, DRAWIT );
		}
	    case 4: 
		makehandicap( bot, top, 0, t, type, DRAWIT );
	    case 3: 
		makehandicap( top, bot, 0, t, type, DRAWIT );
	    case 2: 
		makehandicap( bot, bot, 0, t, type, DRAWIT );
		makehandicap( top, top, 0, t, type, DRAWIT );
		break;
	    default: 
		break;
	}
}

sgf_hcap( f, s, h )
	FILE *f;
	int s, h;
{
	char *ss = NULL;

	switch( s ) {
	    case 19:
		switch( h ) {
		    case 2:
			ss = "AB[dp][pd]";
			break;
		    case 3:
			ss = "AB[dp][pd][dd]";
			break;
		    case 4:
			ss = "AB[dd][dp][pd][pp]";
			break;
		    case 5:
			ss = "AB[dd][dp][jj][pd][pp]";
			break;
		    case 6:
			ss = "AB[dd][dj][dp][pd][pj][pp]";
			break;
		    case 7:
			ss = "AB[dd][dj][dp][jj][pd][pj][pp]";
			break;
		    case 8:
			ss = "AB[dd][dj][dp][jd][jp][pd][pj][pp]";
			break;
		    case 9:
			ss = "AB[dd][dj][dp][jd][jj][jp][pd][pj][pp]";
			break;
		    default:
			break;
		}
		break;
	    case 13:
		switch( h ) {
		    case 2:
			ss = "AB[dj][jd]";
			break;
		    case 3:
			ss = "AB[dj][jd][dd]";
			break;
		    case 4:
			ss = "AB[dd][dj][jd][jj]";
			break;
		    case 5:
			ss = "AB[dd][dj][gg][jd][jj]";
			break;
		    default:
			break;
		}
		break;
	    case 9:
		switch( h ) {
		    case 2:
			ss = "AB[cg][gc]";
			break;
		    case 3:
			ss = "AB[cg][gc][cc]";
			break;
		    case 4:
			ss = "AB[cc][cg][gc][gg]";
			break;
		    default:
			break;
		}
		break;
	    default:
		break;
	}

	if( ss != NULL ) fprintf( f, ";%s\n", ss );
}


 
/*
In case of any problems, please drop me a note, I'll be happy to help. 
I'll happily do further testing of it myself (I am sceptical of it
working without testing, naturally), especially as I got XFree-86 atop
Linux at home and I like using X client, so please let me know if it
seems you have succeeded in inserting it.  I also believe fmc will be
willing to test it. 

Some of the code was originally written by Giovanni Pezzino (giovax@IGS);
he has given consent to giving out the source.  Any and all
comments are welcome. 

	Olli, 3 dan

E-mail: olli.lounela@helsinki.fi          ! .sig still under construction.
Blame me only for any opinions expressed. !  Never you mind, I don't either.
*/

/* defines from igc.h */
 
#define MAXFILTERSZ 100
#define MAXNAME 14		/* This was there before me */

#define FRIEND 1
#define KILL 2

/*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*/

/* global variables from igc.c */

char fltarget[MAXFILTERSZ][MAXNAME];	/* friends list */
int flttypes[MAXFILTERSZ];
int filter_n = 0;
char flnames[][7]={"friend", "kill"};

	/* (...) */

int filterstatus = 0;		/* Filter login messages for friends? */

/*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*/

/* actual friends/kill filter routines from igc.c */

void dosetfilter(first) /* Set filter status to whatever caller wants   */
char *first;            /* ...gotta rewrite this RSN... :-)             */
{
   if (!strcmp(first, "none")) filterstatus = 0;
   if (!strcmp(first, flnames[0])) filterstatus = FRIEND;
   if (!strcmp(first, flnames[1])) filterstatus = KILL;
   if (!strcmp(first, "both")) filterstatus = KILL | FRIEND;
}


void doaddlist(who, filter)     /* Add name to filtering list           */
char *who;		/* or print the list if no arguments given.     */
int filter;
{
   int i, f;
   char buff[60];

                        /* If who argument not given, list the filter   */
   if (!who) {
      if( filterstatus & 1 ) fprintf( stderr, "LOCAL: Friends list is ENABLED\n" );
      else fprintf( stderr, "LOCAL: Friends list is DISABLED\n" );
      if( filterstatus & 2 ) fprintf( stderr, "LOCAL: Kill list is ENABLED\n" );
      else fprintf( stderr, "LOCAL: Kill list is DISABLED\n" );
      fprintf( stderr, "LOCAL: Current filtering list:\n");
      for (i = 0; i < filter_n; i++) {
            fprintf( stderr, "\t%s: %s\n", fltarget[i], flnames[flttypes[i]-1]);
      }
      return;
   }

   if (filter_n >= MAXFILTERSZ)
      fprintf( stderr, "LOCAL: Filtering list is full\n");
   else if (-1 != (i=find_flist(who))) {
      if (flttypes[i] == filter) {
         fprintf( stderr, "LOCAL: Player %s already in %s list\n", who, 
         	 flnames[filter-1]);
         return;
      } else
         flttypes[i]=filter;
   } else {
      strncpy(fltarget[filter_n], who, strlen(who));
      flttypes[filter_n]=filter;
      filter_n++;
   }
   fprintf( stderr, "LOCAL: %s now in %s list\n", who, flnames[filter-1]);
}


void dofriend(who)
char *who;
{
   doaddlist(who, FRIEND);
}


void dokill(who)
char *who;
{
   doaddlist(who, KILL);
}


int isfiltered(text, f) /* Find if text contains a defined filter word  */
char *text;		/* Returns index to fltarget[]			*/
int f;
{
   int i = 0;
   char c, *j, *k;
   extern char *SCstrstr();

   if( dbg ) fprintf( stderr, "ISFILTERED: >>%s<<\n", text );

   for(i=0;i<filter_n;i++) {
                        /* Make sure we check only correct filter       */
      if(!(flttypes[i] & f)) continue;

      k=text;
                        /* Find the substring in text                   */
      while (j=SCstrstr(k, fltarget[i])) {
	 if( f == KILL && j-text>1 ) break;
                        /* Check that the substring starts with         */
                        /* acceptable chars and doesn't start at start  */
                        /* of text (text underflow test)                */
         if(j==text || ((c= *(j-1))=='{' || c==' ' || c=='!' || c=='*'))
                        /* Check that it ends to acceptable chars, too  */
            if((c= *(j+strlen(fltarget[i])))==' ' || c=='!' || c=='*') {
		if( dbg ) fprintf( stderr, "ISFILTERED: return %d\n", i );
                return i;
	    }
         k= ++j;
      }
   }

   if( dbg ) fprintf( stderr, "ISFILTERED: return -1\n" );
   return( (-1) );
}



void doremovelist(who)  /* Remove name from filtering list, no matter   */
char *who;              /* which filter he is on                        */
{
   char i;

   if (!who)
      fprintf( stderr, "LOCAL: This command requires 1 parameter\n");
   else if (-1 != (i = find_flist(who))) {
      for (; i < filter_n; i++) {
         strcpy(fltarget[i], fltarget[i + 1]);
         flttypes[i]=flttypes[i+1];
      }
      filter_n--;
      fprintf( stderr, "LOCAL: Name removed\n");
   } else
      fprintf( stderr, "LOCAL: Name not in list\n");
}


int find_flist(who)
char *who;
{
   char i;

   int x = strlen(who);

   if( who[x-1] == '\n' ) who[x-1] = '\0';

   for (i = 0; i < filter_n && strncmp(fltarget[i], who, x); i++);
   return i == filter_n ? -1 : i;
}


/* Definitions of the routines to call:

void dofriend(string)
char *string;

void dokill(string)
char *string;

	Put a string to appropriate filter list.  If already in one,
	change the filter type to the one requested, or signal error if
	already in that list.  If string is omitted, complete filtering
	list is printed. 

void doremovelist(string)
char *string;

	Remove item *string from any and all lists.  No questions asked.
	(but perhaps some complaints given :-)

void dosetfilter(string)
char *string;

	Set filtering to requested status.  Legal requests are "both",
	"none", and values of filter name strings (flnames[0] and
	flnames[1])

How to use: Parse the commands and pass the parameter to the appropriate
	routine(s), both when reading rc file and when called
	interactively.  You must decide the names for the commands (if
	you wish to change them) and then call these routines after the
	corresponding command is given.  Also, remember to set the
	strings in flnames[] to make it easier for the user remember the
	commands. 

How I have named and defined them in igc internal help:

	filter <type>  Set filtering to friend, kill, both or none
	friend <name>  Add a name to the friends filter list
	kill <name>    Add a name to the kill filter list
	rmfilter <name> Remove a name from all filters

BUGS:	Does not differentiate between reading of rc file and
	interactive use; prints the adding/removing text nevertheless. 

Comments: routines follow the naming convention of igc: all top level
	routines (those that get called by user command) are of the form
	do<whatever> (as is also doaddlist()). 
	Both filters use a single common matrix.
*/

/* end of olli code  =SC */
